@@ -5,6 +5,7 @@
 #include <unistd.h>
 
 #include <osmocom/core/select.h>
+#include <osmocom/core/socket.h>
 #include <osmocom/core/talloc.h>
 #include <osmocom/core/msgb.h>
 #include <osmocom/core/logging.h>
@@ -43,19 +44,28 @@ void sighandler(int foo)
 
 int read_cb(struct osmo_stream_srv *conn)
 {
+	int bytes;
 	struct msgb *msg;
 
-	LOGP(DSTREAMTEST, LOGL_NOTICE, "received message from stream\n");
+	LOGP(DSTREAMTEST, LOGL_NOTICE, "receiving message from stream... ");
 
 	msg = msgb_alloc(1024, "STREAMSERVER/test");
 	if (msg == NULL) {
-		LOGP(DSTREAMTEST, LOGL_ERROR, "cannot allocate message\n");
-		return 0;
-	}
-	if (osmo_stream_srv_recv(conn, msg) < 0) {
-		LOGP(DSTREAMTEST, LOGL_ERROR, "cannot receive message\n");
+		LOGPC(DSTREAMTEST, LOGL_ERROR, "cannot allocate message\n");
 		return 0;
 	}
+
+	bytes = osmo_stream_srv_recv(conn, msg);
+
+	if (bytes <= 0) {
+		if (bytes < 0)
+			LOGPC(DSTREAMTEST, LOGL_ERROR, "cannot receive message: %s\n", strerror(-bytes));
+		else
+			LOGPC(DSTREAMTEST, LOGL_ERROR, "client closed connection\n");
+		osmo_stream_srv_destroy(conn);
+	} else
+		LOGPC(DSTREAMTEST, LOGL_NOTICE, "got %d (%d) bytes: %s\n", bytes, msg->len, msgb_hexdump(msg));
+
 	msgb_free(msg);
 	return 0;
 }
@@ -68,6 +78,8 @@ static int close_cb(struct osmo_stream_srv *dummy)
 
 static int accept_cb(struct osmo_stream_srv_link *srv, int fd)
 {
+	char buf[OSMO_SOCK_NAME_MAXLEN];
+
 	if (conn != NULL) {
 		LOGP(DSTREAMTEST, LOGL_ERROR, "Sorry, this example only "
 			"support one client simultaneously\n");
@@ -82,6 +94,9 @@ static int accept_cb(struct osmo_stream_srv_link *srv, int fd)
 		return -1;
 	}
 
+	osmo_sock_get_name_buf(buf, OSMO_SOCK_NAME_MAXLEN, fd);
+	LOGP(DSTREAMTEST, LOGL_NOTICE, "accepted client: %s\n", buf);
+
 	return 0;
 }
 
@@ -125,6 +140,7 @@ int main(void)
 	msgb_talloc_ctx_init(tall_test, 0);
 	osmo_init_logging2(tall_test, &osmo_stream_srv_test_log_info);
 	log_set_log_level(osmo_stderr_target, 1);
+	log_set_category_filter(osmo_stderr_target, DLINP, 0, LOGL_INFO);
 
 	/*
 	 * initialize stream srv.
@@ -132,7 +148,7 @@ int main(void)
 
 	srv = osmo_stream_srv_link_create(tall_test);
 	if (srv == NULL) {
-		fprintf(stderr, "cannot create client\n");
+		fprintf(stderr, "cannot create server link\n");
 		exit(EXIT_FAILURE);
 	}
 	osmo_stream_srv_link_set_addr(srv, "127.0.0.1");
@@ -140,7 +156,7 @@ int main(void)
 	osmo_stream_srv_link_set_accept_cb(srv, accept_cb);
 
 	if (osmo_stream_srv_link_open(srv) < 0) {
-		fprintf(stderr, "cannot open client\n");
+		fprintf(stderr, "cannot open server link\n");
 		exit(EXIT_FAILURE);
 	}
 
@@ -149,13 +165,10 @@ int main(void)
 		LOGP(DSTREAMTEST, LOGL_ERROR, "OOM\n");
 		exit(EXIT_FAILURE);
 	}
-	kbd_ofd->fd = STDIN_FILENO;
-	kbd_ofd->when = BSC_FD_READ;
-	kbd_ofd->data = srv;
-	kbd_ofd->cb = kbd_cb;
+	osmo_fd_setup(kbd_ofd, STDIN_FILENO, OSMO_FD_READ, kbd_cb, srv, 0);
 	osmo_fd_register(kbd_ofd);
 
-	LOGP(DSTREAMTEST, LOGL_NOTICE, "Entering main loop\n");
+	LOGP(DSTREAMTEST, LOGL_NOTICE, "Entering main loop on %s\n", osmo_stream_srv_link_get_sockname(srv));
 
 	while(1) {
 		osmo_select_main(0);
