@@ -95,7 +95,7 @@ static int osmo_dgram_tx_write(struct osmo_dgram_tx *conn)
 	LOGP(DLINP, LOGL_DEBUG, "sending data\n");
 
 	if (llist_empty(&conn->tx_queue)) {
-		conn->ofd.when &= ~OSMO_FD_WRITE;
+		osmo_fd_write_disable(&conn->ofd);
 		return 0;
 	}
 	lh = conn->tx_queue.next;
@@ -119,7 +119,7 @@ static int osmo_dgram_tx_fd_cb(struct osmo_fd *ofd, unsigned int what)
 		LOGP(DLINP, LOGL_DEBUG, "write\n");
 		osmo_dgram_tx_write(conn);
 	}
-        return 0;
+	return 0;
 }
 
 /*! \brief Create an Osmocom datagram transmitter
@@ -135,11 +135,7 @@ struct osmo_dgram_tx *osmo_dgram_tx_create(void *ctx)
 	if (!conn)
 		return NULL;
 
-	conn->ofd.fd = -1;
-	conn->ofd.when |= OSMO_FD_READ;
-	conn->ofd.priv_nr = 0;	/* XXX */
-	conn->ofd.cb = osmo_dgram_tx_fd_cb;
-	conn->ofd.data = conn;
+	osmo_fd_setup(&conn->ofd, -1, OSMO_FD_READ, osmo_dgram_tx_fd_cb, conn, 0);
 	INIT_LLIST_HEAD(&conn->tx_queue);
 
 	return conn;
@@ -240,7 +236,7 @@ void osmo_dgram_tx_send(struct osmo_dgram_tx *conn,
 				 struct msgb *msg)
 {
 	msgb_enqueue(&conn->tx_queue, msg);
-	conn->ofd.when |= OSMO_FD_WRITE;
+	osmo_fd_write_enable(&conn->ofd);
 }
 
 /*
@@ -250,12 +246,12 @@ void osmo_dgram_tx_send(struct osmo_dgram_tx *conn,
 #define OSMO_DGRAM_RX_F_RECONF	(1 << 0)
 
 struct osmo_dgram_rx {
-        struct osmo_fd                  ofd;
-        char                            *addr;
-        uint16_t                        port;
+	struct osmo_fd		ofd;
+	char			*addr;
+	uint16_t		port;
 	int (*cb)(struct osmo_dgram_rx *conn);
-        void                            *data;
-	unsigned int			flags;
+	void			*data;
+	unsigned int		flags;
 };
 
 /*! \brief Receive data via Osmocom datagram receiver
@@ -309,10 +305,7 @@ struct osmo_dgram_rx *osmo_dgram_rx_create(void *ctx)
 	if (!conn)
 		return NULL;
 
-	conn->ofd.fd = -1;
-	conn->ofd.when |= OSMO_FD_READ;
-	conn->ofd.cb = osmo_dgram_rx_cb;
-	conn->ofd.data = conn;
+	osmo_fd_setup(&conn->ofd, -1, OSMO_FD_READ, osmo_dgram_rx_cb, conn, 0);
 
 	return conn;
 }
