@@ -221,15 +221,15 @@ static int setsockopt_nodelay(int fd, int proto, int on)
  */
 
 enum osmo_stream_cli_state {
-        STREAM_CLI_STATE_CLOSED,	 /* No fd associated, no timer active */
-        STREAM_CLI_STATE_WAIT_RECONNECT, /* No fd associated, has timer active to try to connect again */
-        STREAM_CLI_STATE_CONNECTING,	 /* Fd associated, but connection not yet confirmed by peer or lower layers */
-        STREAM_CLI_STATE_CONNECTED,	 /* Fd associated and connection is established */
-        STREAM_CLI_STATE_MAX
+	STREAM_CLI_STATE_CLOSED,	 /* No fd associated, no timer active */
+	STREAM_CLI_STATE_WAIT_RECONNECT, /* No fd associated, has timer active to try to connect again */
+	STREAM_CLI_STATE_CONNECTING,	 /* Fd associated, but connection not yet confirmed by peer or lower layers */
+	STREAM_CLI_STATE_CONNECTED,	 /* Fd associated and connection is established */
+	STREAM_CLI_STATE_MAX
 };
 
 static const struct value_string stream_cli_state_names[] = {
-	{ STREAM_CLI_STATE_CLOSED,         "CLOSED" },
+	{ STREAM_CLI_STATE_CLOSED,	 "CLOSED" },
 	{ STREAM_CLI_STATE_WAIT_RECONNECT, "WAIT_RECONNECT" },
 	{ STREAM_CLI_STATE_CONNECTING,     "CONNECTING" },
 	{ STREAM_CLI_STATE_CONNECTED,      "CONNECTED" },
@@ -251,11 +251,13 @@ struct osmo_stream_cli {
 	struct osmo_timer_list		timer;
 	enum osmo_stream_cli_state	state;
 	char				*addr[OSMO_STREAM_MAX_ADDRS];
-	uint8_t                         addrcnt;
+	uint8_t			 	addrcnt;
 	uint16_t			port;
 	char				*local_addr[OSMO_STREAM_MAX_ADDRS];
-	uint8_t                         local_addrcnt;
+	uint8_t			 	local_addrcnt;
 	uint16_t			local_port;
+	int				sk_domain;
+	int				sk_type;
 	uint16_t			proto;
 	int (*connect_cb)(struct osmo_stream_cli *srv);
 	int (*disconnect_cb)(struct osmo_stream_cli *srv);
@@ -289,6 +291,7 @@ void osmo_stream_cli_reconnect(struct osmo_stream_cli *cli)
 
 /*! \brief Check if Osmocom Stream Client is in connected state
  *  \param[in] cli Osmocom Stream Client
+ *  \return true if connected, false otherwise
  */
 bool osmo_stream_cli_is_connected(struct osmo_stream_cli *cli)
 {
@@ -334,7 +337,7 @@ static int osmo_stream_cli_write(struct osmo_stream_cli *cli)
 	int ret;
 
 	if (llist_empty(&cli->tx_queue)) {
-		cli->ofd.when &= ~OSMO_FD_WRITE;
+		osmo_fd_write_disable(&cli->ofd);
 		return 0;
 	}
 	lh = cli->tx_queue.next;
@@ -348,20 +351,31 @@ static int osmo_stream_cli_write(struct osmo_stream_cli *cli)
 
 	LOGSCLI(cli, LOGL_DEBUG, "sending %u bytes of data\n", msgb_length(msg));
 
-	switch (cli->proto) {
+	switch (cli->sk_domain) {
+	case AF_UNIX:
+		ret = send(cli->ofd.fd, msg->data, msg->len, 0);
+		break;
+	case AF_UNSPEC:
+	case AF_INET:
+	case AF_INET6:
+		switch (cli->proto) {
 #ifdef HAVE_LIBSCTP
-	case IPPROTO_SCTP:
-		memset(&sinfo, 0, sizeof(sinfo));
-		sinfo.sinfo_ppid = htonl(msgb_sctp_ppid(msg));
-		sinfo.sinfo_stream = msgb_sctp_stream(msg);
-		ret = sctp_send(cli->ofd.fd, msg->data, msgb_length(msg),
-				&sinfo, MSG_NOSIGNAL);
-		break;
+		case IPPROTO_SCTP:
+			memset(&sinfo, 0, sizeof(sinfo));
+			sinfo.sinfo_ppid = htonl(msgb_sctp_ppid(msg));
+			sinfo.sinfo_stream = msgb_sctp_stream(msg);
+			ret = sctp_send(cli->ofd.fd, msg->data, msgb_length(msg),
+					&sinfo, MSG_NOSIGNAL);
+			break;
 #endif
-	case IPPROTO_TCP:
-	default:
-		ret = send(cli->ofd.fd, msg->data, msgb_length(msg), 0);
+		case IPPROTO_TCP:
+		default:
+			ret = send(cli->ofd.fd, msg->data, msgb_length(msg), 0);
+			break;
+		}
 		break;
+	default:
+		ret = -ENOTSUP;
 	}
 	if (ret < 0) {
 		if (errno == EPIPE || errno == ENOTCONN) {
@@ -373,6 +387,20 @@ static int osmo_stream_cli_write(struct osmo_stream_cli *cli)
 	return 0;
 }
 
+static int _setsockopt_nosigpipe(struct osmo_stream_cli *cli)
+{
+#ifdef SO_NOSIGPIPE
+	int ret;
+	int val = 1;
+	ret = setsockopt(cli->ofd.fd, SOL_SOCKET, SO_NOSIGPIPE, (void *)&val, sizeof(val));
+	if (ret < 0)
+		LOGSCLI(cli, LOGL_DEBUG, "Failed setting SO_NOSIGPIPE: %s\n", strerror(errno));
+	return ret;
+#else
+	return 0;
+#endif
+}
+
 static int osmo_stream_cli_fd_cb(struct osmo_fd *ofd, unsigned int what)
 {
 	struct osmo_stream_cli *cli = ofd->data;
@@ -390,19 +418,24 @@ static int osmo_stream_cli_fd_cb(struct osmo_fd *ofd, unsigned int what)
 		/* If messages got enqueued while 'connecting', keep WRITE flag
 		   up to dispatch them upon next main loop step */
 		if (llist_empty(&cli->tx_queue))
-			cli->ofd.when &= ~OSMO_FD_WRITE;
+			osmo_fd_write_disable(&cli->ofd);
 
 		LOGSCLI(cli, LOGL_DEBUG, "connection done.\n");
 		cli->state = STREAM_CLI_STATE_CONNECTED;
-		if (cli->proto == IPPROTO_SCTP) {
-#ifdef SO_NOSIGPIPE
-			int val = 1;
-
-			ret = setsockopt(ofd->fd, SOL_SOCKET, SO_NOSIGPIPE, (void*)&val, sizeof(val));
-			if (ret < 0)
-				LOGSCLI(cli, LOGL_DEBUG, "Failed setting SO_NOSIGPIPE: %s\n", strerror(errno));
-#endif
-			sctp_sock_activate_events(ofd->fd);
+		switch (cli->sk_domain) {
+		case AF_UNIX:
+			_setsockopt_nosigpipe(cli);
+			break;
+		case AF_UNSPEC:
+		case AF_INET:
+		case AF_INET6:
+			if (cli->proto == IPPROTO_SCTP) {
+				_setsockopt_nosigpipe(cli);
+				sctp_sock_activate_events(ofd->fd);
+			}
+			break;
+		default:
+			break;
 		}
 		if (cli->connect_cb)
 			cli->connect_cb(cli);
@@ -421,7 +454,7 @@ static int osmo_stream_cli_fd_cb(struct osmo_fd *ofd, unsigned int what)
 		/* Only CONNECTING and CONNECTED states are expected, since they are the only states where FD exists: */
 		osmo_panic("osmo_stream_cli_fd_cb called with unexpected state %d\n", cli->state);
 	}
-        return 0;
+	return 0;
 }
 
 static void cli_timer_cb(void *data);
@@ -429,7 +462,9 @@ static void cli_timer_cb(void *data);
 /*! \brief Create an Osmocom stream client
  *  \param[in] ctx talloc context from which to allocate memory
  *  This function allocates a new \ref osmo_stream_cli and initializes
- *  it with default values (5s reconnect timer, TCP protocol) */
+ *  it with default values (5s reconnect timer, TCP protocol)
+ *  \return allocated stream client, or NULL in case of error
+ */
 struct osmo_stream_cli *osmo_stream_cli_create(void *ctx)
 {
 	struct osmo_stream_cli *cli;
@@ -438,6 +473,8 @@ struct osmo_stream_cli *osmo_stream_cli_create(void *ctx)
 	if (!cli)
 		return NULL;
 
+	cli->sk_domain = AF_UNSPEC;
+	cli->sk_type = SOCK_STREAM;
 	cli->proto = IPPROTO_TCP;
 	cli->ofd.fd = -1;
 	cli->ofd.priv_nr = 0;	/* XXX */
@@ -554,6 +591,46 @@ osmo_stream_cli_set_proto(struct osmo_stream_cli *cli, uint16_t proto)
 	cli->flags |= OSMO_STREAM_CLI_F_RECONF;
 }
 
+/*! \brief Set the socket type for the stream server link
+ *  \param[in] cli Stream Client to modify
+ *  \param[in] type Socket Type (like SOCK_STREAM (default), SOCK_SEQPACKET, ...)
+ *  \returns zero on success, negative on error.
+ */
+int osmo_stream_cli_set_type(struct osmo_stream_cli *cli, int type)
+{
+	switch (type) {
+	case SOCK_STREAM:
+	case SOCK_SEQPACKET:
+		break;
+	default:
+		return -ENOTSUP;
+	}
+	cli->sk_type = type;
+	cli->flags |= OSMO_STREAM_CLI_F_RECONF;
+	return 0;
+}
+
+/*! \brief Set the socket type for the stream server link
+ *  \param[in] cli Stream Client to modify
+ *  \param[in] type Socket Domain (like AF_UNSPEC (default for IP), AF_UNIX, AF_INET, ...)
+ *  \returns zero on success, negative on error.
+ */
+int osmo_stream_cli_set_domain(struct osmo_stream_cli *cli, int domain)
+{
+	switch (domain) {
+	case AF_UNSPEC:
+	case AF_INET:
+	case AF_INET6:
+	case AF_UNIX:
+		break;
+	default:
+		return -ENOTSUP;
+	}
+	cli->sk_domain = domain;
+	cli->flags |= OSMO_STREAM_CLI_F_RECONF;
+	return 0;
+}
+
 /*! \brief Set the reconnect time of the stream client socket
  *  \param[in] cli Stream Client to modify
  *  \param[in] timeout Re-connect timeout in seconds or negative value to disable auto-reconnection */
@@ -644,6 +721,7 @@ void osmo_stream_cli_destroy(struct osmo_stream_cli *cli)
  * Open connection of an Osmocom stream client
  *  \param[in] cli Stream Client to connect
  *  \param[in] reconect 1 if we should not automatically reconnect
+ *  \return negative on error, 0 on success
  */
 int osmo_stream_cli_open2(struct osmo_stream_cli *cli, int reconnect)
 {
@@ -658,14 +736,14 @@ int osmo_stream_cli_open2(struct osmo_stream_cli *cli, int reconnect)
 	switch (cli->proto) {
 #ifdef HAVE_LIBSCTP
 	case IPPROTO_SCTP:
-		ret = osmo_sock_init2_multiaddr(AF_INET, SOCK_STREAM, cli->proto,
+		ret = osmo_sock_init2_multiaddr(AF_UNSPEC, SOCK_STREAM, cli->proto,
 						(const char **)cli->local_addr, cli->local_addrcnt, cli->local_port,
 						(const char **)cli->addr, cli->addrcnt, cli->port,
 						OSMO_SOCK_F_CONNECT|OSMO_SOCK_F_BIND|OSMO_SOCK_F_NONBLOCK);
 		break;
 #endif
 	default:
-		ret = osmo_sock_init2(AF_INET, SOCK_STREAM, cli->proto,
+		ret = osmo_sock_init2(AF_UNSPEC, SOCK_STREAM, cli->proto,
 				      cli->local_addr[0], cli->local_port,
 				      cli->addr[0], cli->port,
 				      OSMO_SOCK_F_CONNECT|OSMO_SOCK_F_BIND|OSMO_SOCK_F_NONBLOCK);
@@ -676,8 +754,7 @@ int osmo_stream_cli_open2(struct osmo_stream_cli *cli, int reconnect)
 			osmo_stream_cli_reconnect(cli);
 		return ret;
 	}
-	cli->ofd.fd = ret;
-	cli->ofd.when = OSMO_FD_READ | OSMO_FD_WRITE;
+	osmo_fd_setup(&cli->ofd, ret, OSMO_FD_READ | OSMO_FD_WRITE, cli->ofd.cb, cli->ofd.data, cli->ofd.priv_nr);
 
 	if (cli->flags & OSMO_STREAM_CLI_F_NODELAY) {
 		ret = setsockopt_nodelay(cli->ofd.fd, cli->proto, 1);
@@ -716,7 +793,8 @@ void osmo_stream_cli_set_nodelay(struct osmo_stream_cli *cli, bool nodelay)
 /*! \brief Open connection of an Osmocom stream client
  *  By default the client will automatically reconnect after default timeout.
  *  To disable this, use osmo_stream_cli_set_reconnect_timeout() before calling this function.
- *  \param[in] cli Stream Client to connect */
+ *  \param[in] cli Stream Client to connect
+ *  \return negative on error, 0 on success */
 int osmo_stream_cli_open(struct osmo_stream_cli *cli)
 {
 	int ret;
@@ -727,29 +805,38 @@ int osmo_stream_cli_open(struct osmo_stream_cli *cli)
 
 	cli->flags &= ~OSMO_STREAM_CLI_F_RECONF;
 
-
-	switch (cli->proto) {
-#ifdef HAVE_LIBSCTP
-	case IPPROTO_SCTP:
-		ret = osmo_sock_init2_multiaddr(AF_INET, SOCK_STREAM, cli->proto,
-						(const char **)cli->local_addr, cli->local_addrcnt, cli->local_port,
-						(const char **)cli->addr, cli->addrcnt, cli->port,
-						OSMO_SOCK_F_CONNECT|OSMO_SOCK_F_BIND|OSMO_SOCK_F_NONBLOCK);
+	switch (cli->sk_domain) {
+	case AF_UNIX:
+		ret = osmo_sock_unix_init(cli->sk_type, 0, cli->addr[0], OSMO_SOCK_F_CONNECT|OSMO_SOCK_F_BIND|OSMO_SOCK_F_NONBLOCK);
 		break;
+	case AF_INET:
+	case AF_INET6:
+	case AF_UNSPEC:
+		switch (cli->proto) {
+#ifdef HAVE_LIBSCTP
+		case IPPROTO_SCTP:
+			ret = osmo_sock_init2_multiaddr(cli->sk_domain, cli->sk_type, cli->proto,
+							(const char **)cli->local_addr, cli->local_addrcnt, cli->local_port,
+							(const char **)cli->addr, cli->addrcnt, cli->port,
+							OSMO_SOCK_F_CONNECT|OSMO_SOCK_F_BIND|OSMO_SOCK_F_NONBLOCK);
+			break;
 #endif
+		default:
+			ret = osmo_sock_init2(cli->sk_domain, cli->sk_type, cli->proto,
+					      cli->local_addr[0], cli->local_port,
+					      cli->addr[0], cli->port,
+					      OSMO_SOCK_F_CONNECT|OSMO_SOCK_F_BIND|OSMO_SOCK_F_NONBLOCK);
+		}
+		break;
 	default:
-		ret = osmo_sock_init2(AF_INET, SOCK_STREAM, cli->proto,
-				      cli->local_addr[0], cli->local_port,
-				      cli->addr[0], cli->port,
-				      OSMO_SOCK_F_CONNECT|OSMO_SOCK_F_BIND|OSMO_SOCK_F_NONBLOCK);
+		return -ENOTSUP;
 	}
 
 	if (ret < 0) {
 		osmo_stream_cli_reconnect(cli);
 		return ret;
 	}
-	cli->ofd.fd = ret;
-	cli->ofd.when = OSMO_FD_READ | OSMO_FD_WRITE;
+	osmo_fd_setup(&cli->ofd, ret, OSMO_FD_READ | OSMO_FD_WRITE, cli->ofd.cb, cli->ofd.data, cli->ofd.priv_nr);
 
 	if (cli->flags & OSMO_STREAM_CLI_F_NODELAY) {
 		ret = setsockopt_nodelay(cli->ofd.fd, cli->proto, 1);
@@ -784,7 +871,7 @@ static void cli_timer_cb(void *data)
 void osmo_stream_cli_send(struct osmo_stream_cli *cli, struct msgb *msg)
 {
 	msgb_enqueue(&cli->tx_queue, msg);
-	cli->ofd.when |= OSMO_FD_WRITE;
+	osmo_fd_write_enable(&cli->ofd);
 }
 
 /*! \brief Receive data via an Osmocom stream client
@@ -820,21 +907,25 @@ int osmo_stream_cli_recv(struct osmo_stream_cli *cli, struct msgb *msg)
 #define OSMO_STREAM_SRV_F_NODELAY	(1 << 1)
 
 struct osmo_stream_srv_link {
-        struct osmo_fd                  ofd;
-        char                            *addr[OSMO_STREAM_MAX_ADDRS];
-        uint8_t                         addrcnt;
-        uint16_t                        port;
-        uint16_t                        proto;
-        int (*accept_cb)(struct osmo_stream_srv_link *srv, int fd);
-        void                            *data;
-	int				flags;
+	struct osmo_fd		ofd;
+	char			*addr[OSMO_STREAM_MAX_ADDRS];
+	uint8_t			addrcnt;
+	uint16_t		port;
+	int			sk_domain;
+	int			sk_type;
+	uint16_t		proto;
+	int (*accept_cb)(struct osmo_stream_srv_link *srv, int fd);
+	void			*data;
+	int			flags;
 };
 
 static int osmo_stream_srv_fd_cb(struct osmo_fd *ofd, unsigned int what)
 {
 	int ret;
 	int sock_fd;
-	struct sockaddr_in sa;
+	char addrstr[128];
+	bool is_ipv6 = false;
+	struct sockaddr_storage sa;
 	socklen_t sa_len = sizeof(sa);
 	struct osmo_stream_srv_link *link = ofd->data;
 
@@ -844,14 +935,35 @@ static int osmo_stream_srv_fd_cb(struct osmo_fd *ofd, unsigned int what)
 			"peer, reason=`%s'\n", strerror(errno));
 		return ret;
 	}
-	LOGP(DLINP, LOGL_DEBUG, "accept()ed new link from %s to port %u\n",
-		inet_ntoa(sa.sin_addr), link->port);
 	sock_fd = ret;
 
-	if (link->proto == IPPROTO_SCTP) {
-		ret = sctp_sock_activate_events(sock_fd);
-		if (ret < 0)
-			goto error_close_socket;
+	is_ipv6 = false;
+	switch (((struct sockaddr *)&sa)->sa_family) {
+	case AF_UNIX:
+		LOGP(DLINP, LOGL_DEBUG, "accept()ed new link on fd %d\n",
+		     sock_fd);
+		break;
+	case AF_INET6:
+		is_ipv6 = true;
+		/* fall through */
+	case AF_INET:
+		LOGP(DLINP, LOGL_DEBUG, "accept()ed new link from %s to port %u\n",
+			inet_ntop(is_ipv6 ? AF_INET6 : AF_INET,
+				  is_ipv6 ? (void *)&(((struct sockaddr_in6 *)&sa)->sin6_addr) :
+					    (void *)&(((struct sockaddr_in *)&sa)->sin_addr),
+				  addrstr, sizeof(addrstr)),
+			link->port);
+
+		if (link->proto == IPPROTO_SCTP) {
+			ret = sctp_sock_activate_events(sock_fd);
+			if (ret < 0)
+				goto error_close_socket;
+		}
+		break;
+	default:
+		LOGP(DLINP, LOGL_DEBUG, "accept()ed unexpected address family %d\n",
+		     ((struct sockaddr *)&sa)->sa_family);
+		goto error_close_socket;
 	}
 
 	if (link->flags & OSMO_STREAM_SRV_F_NODELAY) {
@@ -889,11 +1001,10 @@ struct osmo_stream_srv_link *osmo_stream_srv_link_create(void *ctx)
 	if (!link)
 		return NULL;
 
+	link->sk_domain = AF_UNSPEC;
+	link->sk_type = SOCK_STREAM;
 	link->proto = IPPROTO_TCP;
-	link->ofd.fd = -1;
-	link->ofd.when |= OSMO_FD_READ | OSMO_FD_WRITE;
-	link->ofd.cb = osmo_stream_srv_fd_cb;
-	link->ofd.data = link;
+	osmo_fd_setup(&link->ofd, -1, OSMO_FD_READ | OSMO_FD_WRITE, osmo_stream_srv_fd_cb, link, 0);
 
 	return link;
 }
@@ -971,6 +1082,47 @@ osmo_stream_srv_link_set_proto(struct osmo_stream_srv_link *link,
 	link->flags |= OSMO_STREAM_SRV_F_RECONF;
 }
 
+
+/*! \brief Set the socket type for the stream server link
+ *  \param[in] link Stream Server Link to modify
+ *  \param[in] type Socket Type (like SOCK_STREAM (default), SOCK_SEQPACKET, ...)
+ *  \returns zero on success, negative on error.
+ */
+int osmo_stream_srv_link_set_type(struct osmo_stream_srv_link *link, int type)
+{
+	switch (type) {
+	case SOCK_STREAM:
+	case SOCK_SEQPACKET:
+		break;
+	default:
+		return -ENOTSUP;
+	}
+	link->sk_type = type;
+	link->flags |= OSMO_STREAM_SRV_F_RECONF;
+	return 0;
+}
+
+/*! \brief Set the socket type for the stream server link
+ *  \param[in] link Stream Server Link to modify
+ *  \param[in] type Socket Domain (like AF_UNSPEC (default for IP), AF_UNIX, AF_INET, ...)
+ *  \returns zero on success, negative on error.
+ */
+int osmo_stream_srv_link_set_domain(struct osmo_stream_srv_link *link, int domain)
+{
+	switch (domain) {
+	case AF_UNSPEC:
+	case AF_INET:
+	case AF_INET6:
+	case AF_UNIX:
+		break;
+	default:
+		return -ENOTSUP;
+	}
+	link->sk_domain = domain;
+	link->flags |= OSMO_STREAM_SRV_F_RECONF;
+	return 0;
+}
+
 /*! \brief Set application private data of the stream server link
  *  \param[in] link Stream Server Link to modify
  *  \param[in] data User-specific data (available in call-back functions) */
@@ -1037,7 +1189,8 @@ void osmo_stream_srv_link_destroy(struct osmo_stream_srv_link *link)
 
 /*! \brief Open the stream server link.  This actually initializes the
  *  underlying socket and binds it to the configured ip/port
- *  \param[in] link Stream Server Link to open */
+ *  \param[in] link Stream Server Link to open
+ *  \return negative on error, 0 on success */
 int osmo_stream_srv_link_open(struct osmo_stream_srv_link *link)
 {
 	int ret;
@@ -1052,17 +1205,28 @@ int osmo_stream_srv_link_open(struct osmo_stream_srv_link *link)
 
 	link->flags &= ~OSMO_STREAM_SRV_F_RECONF;
 
-	switch (link->proto) {
-#ifdef HAVE_LIBSCTP
-	case IPPROTO_SCTP:
-		ret = osmo_sock_init2_multiaddr(AF_INET, SOCK_STREAM, link->proto,
-						(const char **)link->addr, link->addrcnt, link->port,
-						NULL, 0, 0, OSMO_SOCK_F_BIND);
+	switch (link->sk_domain) {
+	case AF_UNIX:
+		ret = osmo_sock_unix_init(link->sk_type, 0, link->addr[0], OSMO_SOCK_F_BIND);
 		break;
+	case AF_UNSPEC:
+	case AF_INET:
+	case AF_INET6:
+		switch (link->proto) {
+#ifdef HAVE_LIBSCTP
+		case IPPROTO_SCTP:
+			ret = osmo_sock_init2_multiaddr(link->sk_domain, link->sk_type, link->proto,
+							(const char **)link->addr, link->addrcnt, link->port,
+							NULL, 0, 0, OSMO_SOCK_F_BIND);
+			break;
 #endif
+		default:
+			ret = osmo_sock_init(link->sk_domain, link->sk_type, link->proto,
+					     link->addr[0], link->port, OSMO_SOCK_F_BIND);
+		}
+		break;
 	default:
-		ret = osmo_sock_init(AF_INET, SOCK_STREAM, link->proto,
-					link->addr[0], link->port, OSMO_SOCK_F_BIND);
+		ret = -ENOTSUP;
 	}
 	if (ret < 0)
 		return ret;
@@ -1092,11 +1256,11 @@ void osmo_stream_srv_link_close(struct osmo_stream_srv_link *link)
 
 struct osmo_stream_srv {
 	struct osmo_stream_srv_link	*srv;
-        struct osmo_fd                  ofd;
-        struct llist_head               tx_queue;
-        int (*closed_cb)(struct osmo_stream_srv *peer);
-        int (*cb)(struct osmo_stream_srv *peer);
-        void                            *data;
+	struct osmo_fd			ofd;
+	struct llist_head		tx_queue;
+	int (*closed_cb)(struct osmo_stream_srv *peer);
+	int (*cb)(struct osmo_stream_srv *peer);
+	void				*data;
 	int				flags;
 };
 
@@ -1129,27 +1293,38 @@ static void osmo_stream_srv_write(struct osmo_stream_srv *conn)
 	LOGP(DLINP, LOGL_DEBUG, "sending data\n");
 
 	if (llist_empty(&conn->tx_queue)) {
-		conn->ofd.when &= ~OSMO_FD_WRITE;
+		osmo_fd_write_disable(&conn->ofd);
 		return;
 	}
 	lh = conn->tx_queue.next;
 	llist_del(lh);
 	msg = llist_entry(lh, struct msgb, list);
 
-	switch (conn->srv->proto) {
-#ifdef HAVE_LIBSCTP
-	case IPPROTO_SCTP:
-		memset(&sinfo, 0, sizeof(sinfo));
-		sinfo.sinfo_ppid = htonl(msgb_sctp_ppid(msg));
-		sinfo.sinfo_stream = msgb_sctp_stream(msg);
-		ret = sctp_send(conn->ofd.fd, msg->data, msgb_length(msg),
-				&sinfo, MSG_NOSIGNAL);
-		break;
-#endif
-	case IPPROTO_TCP:
-	default:
+	switch (conn->srv->sk_domain) {
+	case AF_UNIX:
 		ret = send(conn->ofd.fd, msg->data, msg->len, 0);
 		break;
+	case AF_INET:
+	case AF_INET6:
+	case AF_UNSPEC:
+		switch (conn->srv->proto) {
+#ifdef HAVE_LIBSCTP
+		case IPPROTO_SCTP:
+			memset(&sinfo, 0, sizeof(sinfo));
+			sinfo.sinfo_ppid = htonl(msgb_sctp_ppid(msg));
+			sinfo.sinfo_stream = msgb_sctp_stream(msg);
+			ret = sctp_send(conn->ofd.fd, msg->data, msgb_length(msg),
+					&sinfo, MSG_NOSIGNAL);
+			break;
+#endif
+		case IPPROTO_TCP:
+		default:
+			ret = send(conn->ofd.fd, msg->data, msg->len, 0);
+			break;
+		}
+		break;
+	default:
+		ret = -ENOTSUP;
 	}
 	if (ret < 0) {
 		LOGP(DLINP, LOGL_ERROR, "error to send\n");
@@ -1193,10 +1368,7 @@ osmo_stream_srv_create(void *ctx, struct osmo_stream_srv_link *link,
 		return NULL;
 	}
 	conn->srv = link;
-	conn->ofd.fd = fd;
-	conn->ofd.data = conn;
-	conn->ofd.cb = osmo_stream_srv_cb;
-	conn->ofd.when = OSMO_FD_READ;
+	osmo_fd_setup(&conn->ofd, fd, OSMO_FD_READ, osmo_stream_srv_cb, conn, 0);
 	conn->cb = cb;
 	conn->closed_cb = closed_cb;
 	conn->data = data;
@@ -1282,9 +1454,59 @@ void osmo_stream_srv_send(struct osmo_stream_srv *conn, struct msgb *msg)
 	}
 
 	msgb_enqueue(&conn->tx_queue, msg);
-	conn->ofd.when |= OSMO_FD_WRITE;
+	osmo_fd_write_enable(&conn->ofd);
 }
 
+#ifdef HAVE_LIBSCTP
+static int _sctp_recvmsg_wrapper(int fd, struct msgb *msg)
+{
+	struct sctp_sndrcvinfo sinfo;
+	int flags = 0;
+	int ret;
+
+	ret = sctp_recvmsg(fd, msgb_data(msg), msgb_tailroom(msg),
+			NULL, NULL, &sinfo, &flags);
+	if (flags & MSG_NOTIFICATION) {
+		union sctp_notification *notif = (union sctp_notification *)msgb_data(msg);
+		LOGP(DLINP, LOGL_DEBUG, "NOTIFICATION %u flags=0x%x\n", notif->sn_header.sn_type, notif->sn_header.sn_flags);
+		switch (notif->sn_header.sn_type) {
+		case SCTP_ASSOC_CHANGE:
+			LOGP(DLINP, LOGL_DEBUG, "===> ASSOC CHANGE:");
+			switch (notif->sn_assoc_change.sac_state) {
+			case SCTP_COMM_UP:
+				LOGPC(DLINP, LOGL_DEBUG, " UP\n");
+				break;
+			case SCTP_COMM_LOST:
+				LOGPC(DLINP, LOGL_DEBUG, " LOST\n");
+				break;
+			case SCTP_RESTART:
+				LOGPC(DLINP, LOGL_DEBUG, " RESTART\n");
+				break;
+			case SCTP_SHUTDOWN_COMP:
+				LOGPC(DLINP, LOGL_DEBUG, " SHUTDOWN COMP\n");
+				break;
+			case SCTP_CANT_STR_ASSOC:
+				LOGPC(DLINP, LOGL_DEBUG, " CANT STR ASSOC\n");
+				break;
+			}
+			break;
+		case SCTP_PEER_ADDR_CHANGE:
+			LOGP(DLINP, LOGL_DEBUG, "===> PEER ADDR CHANGE\n");
+			break;
+		case SCTP_SHUTDOWN_EVENT:
+			LOGP(DLINP, LOGL_DEBUG, "===> SHUTDOWN EVT\n");
+			/* Handle this like a regular disconnect */
+			return 0;
+			break;
+		}
+		return -EAGAIN;
+	}
+	msgb_sctp_ppid(msg) = ntohl(sinfo.sinfo_ppid);
+	msgb_sctp_stream(msg) = sinfo.sinfo_stream;
+	return ret;
+}
+#endif
+
 /*! \brief Receive data via Osmocom stream server
  *  \param[in] conn Stream Server from which to receive
  *  \param msg pre-allocate message buffer to which received data is appended
@@ -1292,63 +1514,32 @@ void osmo_stream_srv_send(struct osmo_stream_srv *conn, struct msgb *msg)
  */
 int osmo_stream_srv_recv(struct osmo_stream_srv *conn, struct msgb *msg)
 {
-#ifdef HAVE_LIBSCTP
-	struct sctp_sndrcvinfo sinfo;
-	int flags = 0;
-#endif
 	int ret;
 
 	if (!msg)
 		return -EINVAL;
 
-	switch (conn->srv->proto) {
-#ifdef HAVE_LIBSCTP
-	case IPPROTO_SCTP:
-		ret = sctp_recvmsg(conn->ofd.fd, msgb_data(msg), msgb_tailroom(msg),
-				NULL, NULL, &sinfo, &flags);
-		if (flags & MSG_NOTIFICATION) {
-			union sctp_notification *notif = (union sctp_notification *) msgb_data(msg);
-			LOGP(DLINP, LOGL_DEBUG, "NOTIFICATION %u flags=0x%x\n", notif->sn_header.sn_type, notif->sn_header.sn_flags);
-			switch (notif->sn_header.sn_type) {
-			case SCTP_ASSOC_CHANGE:
-				LOGP(DLINP, LOGL_DEBUG, "===> ASSOC CHANGE:");
-				switch (notif->sn_assoc_change.sac_state) {
-				case SCTP_COMM_UP:
-					LOGPC(DLINP, LOGL_DEBUG, " UP\n");
-					break;
-				case SCTP_COMM_LOST:
-					LOGPC(DLINP, LOGL_DEBUG, " LOST\n");
-					break;
-				case SCTP_RESTART:
-					LOGPC(DLINP, LOGL_DEBUG, " RESTART\n");
-					break;
-				case SCTP_SHUTDOWN_COMP:
-					LOGPC(DLINP, LOGL_DEBUG, " SHUTDOWN COMP\n");
-					break;
-				case SCTP_CANT_STR_ASSOC:
-					LOGPC(DLINP, LOGL_DEBUG, " CANT STR ASSOC\n");
-					break;
-				}
-				break;
-			case SCTP_PEER_ADDR_CHANGE:
-				LOGP(DLINP, LOGL_DEBUG, "===> PEER ADDR CHANGE\n");
-				break;
-			case SCTP_SHUTDOWN_EVENT:
-				LOGP(DLINP, LOGL_DEBUG, "===> SHUTDOWN EVT\n");
-				/* Handle this like a regular disconnect */
-				return 0;
-				break;
-			}
-			return -EAGAIN;
-		}
-		msgb_sctp_ppid(msg) = ntohl(sinfo.sinfo_ppid);
-		msgb_sctp_stream(msg) = sinfo.sinfo_stream;
-		break;
-#endif
-	case IPPROTO_TCP:
-	default:
+	switch (conn->srv->sk_domain) {
+	case AF_UNIX:
 		ret = recv(conn->ofd.fd, msgb_data(msg), msgb_tailroom(msg), 0);
 		break;
+	case AF_INET:
+	case AF_INET6:
+	case AF_UNSPEC:
+		switch (conn->srv->proto) {
+#ifdef HAVE_LIBSCTP
+		case IPPROTO_SCTP:
+			ret = _sctp_recvmsg_wrapper(conn->ofd.fd, msg);
+			break;
+#endif
+		case IPPROTO_TCP:
+		default:
+			ret = recv(conn->ofd.fd, msgb_data(msg), msgb_tailroom(msg), 0);
+			break;
+		}
+		break;
+	default:
+		ret = -ENOTSUP;
 	}
 
 	if (ret < 0) {
