@@ -797,7 +797,7 @@ int main(int argc, char **argv)
 	void *tall_ctx = talloc_named_const(NULL, 1, "Root context");
 	msgb_talloc_ctx_init(tall_ctx, 0);
 	osmo_init_logging2(tall_ctx, &jibuf_test_log_info);
-	log_set_print_filename(osmo_stderr_target, 0);
+	log_set_print_filename2(osmo_stderr_target, LOG_FILENAME_NONE);
 	log_set_log_level(osmo_stderr_target, LOGL_DEBUG);
 	log_set_category_filter(osmo_stderr_target, DLJIBUF, 1, LOGL_DEBUG);
 
